#! /bin/sh
# Paul Sladen 2004-09-23

# Revision 0.0.9 2004-12-16  don't show module extensions
# Revision 0.0.8 2004-12-14  extra string CPU id strings
# Revision 0.0.7 2004-12-01  string comparision with '==' is a bashism
# Revision 0.0.6 2004-12-01  PIII Coppermine test, bug #4262

# Class    Family Model Stepping  Mhz  Mhz String
# PIII          6     8        6  850      "Pentium III (Coppermine)"                         # Jean Privat
# PIII          6    11        1 1131 ---- "Intel(R) Pentium(R) III CPU             1133MHz"  # pandora
# PIII-M        6    11        1 1131| 731 "Intel(R) Pentium(R) III Mobile CPU      1133MHz"  # emeritus
# PIII-M        6    11        1  995   ?? "Intel(R) Pentium(R) III Mobile CPU      1000MHz"  # Pascal Hakim
# PIV          15     2        4 1616   ?? "Intel(R) Pentium(R) 4 CPU 1.60GHz"                # dasboot
# PIV-M        15     2        4            Intel(R) Pentium(R) 4 Mobile CPU 1.70GHz          # Bug 1444
# PIV Xeon     15     2        4 2188   ?? "Intel(R) XEON(TM) CPU 2.20GHz"                    # Googled
# P4-M         15     2        7 2188-1196 "Mobile Intel(R) Pentium(R) 4 - M CPU 2.20GHz"     # Dafydd
# Athlon        6     4        4  750  --- "AMD Athlon(tm) processor"                         # yukidoko (Mako)
# Athlon64     15     4       10 3400?  ?? "AMD Athlon 64 Processor 3400+"                    # iDunno
# Sempron      15     8        2 1600  800 "Mobile AMD Sempron(tm) Processor 2800+"           # Matthew Trent (Ubuntu #1444)

# Code below contains cases for wildcarded matches of:
#
#    Intel(R) Pentium(R) III Mobile CPU
#    Mobile Intel(R) Pentium(R) III CPU - M
#    Pentium III (Coppermine)
#    Intel(R) Pentium(R) M processor
#    Mobile Intel(R) Pentium(R) 4 - M CPU
#    Mobile Intel(R) Celeron(R) CPU
#    mobile AMD Athlon(tm) XP
#    Mobile AMD Athlon(tm) XP
#    AMD Athlon 64 Processor
#    AMD Athlon(tm) 64 Processor

# No .o/.ko module extensions, for feeding the output to 'modprobe' or '/etc/modules'
EXT=''

#are we on a laptop?
LAPTOP=0
if /usr/sbin/laptop-detect; then LAPTOP=1; fi
CPUINFO=/proc/cpuinfo
IOPORTS=/proc/ioports

if [ ! -f $CPUINFO ] ; then
    echo $CPUINFO not detected... >2
    exit 1
fi

MODEL_NAME=`grep '^model name' "$CPUINFO" | head -1 | sed -e 's/^.*: //;'`
CPU=`grep -E '^cpud[^:]+:' "$CPUINFO" | head -1 | sed -e 's/^.*: //;'`
VENDOR_ID=`grep -E '^vendor_id[^:]+:' "$CPUINFO" | head -1 | sed -e 's/^.*: //;'`

MODULE=none
OPTIONS=''

# Too modules for PIII-M depending the chipset.
# modprobe speedstep-ich$EXT || modprobe speestep-smi$EXT  would be another way
if [ -f $IOPORTS ] && grep -q 'Intel ICH' $IOPORTS ; then
  PIII_MODULE=speedstep-ich$EXT
else
  PIII_MODULE=speedstep-smi$EXT
fi

# Noodles: acpi
# Intel(R) Pentium(R) III Mobile CPU

#CyrixInstead
#GenuineIntel
#CentaurHauls
#AuthenticAMD

case "$VENDOR_ID" in
    GenuineIntel*)
    # x86 machines seem to be happy with /proc/cpuinfo 'model name:'
    case "$MODEL_NAME" in
	### Intel Pentium III diveratives
	# Need to speedstep-ich.o on ICH-2/ICH-3 machines (check if on 440-somethingX southbridge) -mjg59
	# Load speedstep-smi otherwise
	# If coppermine, need to pass "coppermine=1" to the module on load (or something?!) -- FIXME this appears not to be correct:
        #   speedstep_smi: Unknown parameter `coppermine'
	# Rumour has it that there are sub-600Mhz PIII Mobiles's (*without* Speedstep)
	# ...and there's also Deschutes(?) PII's that have speedstep
	Intel\(R\)\ Pentium\(R\)\ III\ Mobile\ CPU*)
	    MODULE=$PIII_MODULE ;;

	# JD: says this works with   cpufreq_userspace
	Mobile\ Intel\(R\)\ Pentium\(R\)\ III\ CPU\ -\ M*)
	    MODULE=$PIII_MODULE ;;

	# https://bugzilla.ubuntu.com/show_bug.cgi?id=4262
	# UNCONFIRMED
	Pentium\ III\ \(Coppermine\)*)
	    MODULE=$PIII_MODULE 
	    ;;

	### Intel Pentium 4 / Centrino diveratives
	# jdub wants speedstep_lib.o  (?)
	Intel\(R\)\ Pentium\(R\)\ M\ processor*)
	    MODULE=speedstep-centrino$EXT ;;

	# Unconfirmed except for the cpuid string; from Kirsch on #ubuntu
	Mobile\ Intel\(R\)\ Pentium\(R\)\ 4\ -\ M\ CPU*)
	    MODULE=speedstep-centrino$EXT ;;

        Intel\(R\)\ Pentium\(R\)\ 4\ Mobile\ CPU*)
	    MODULE=speedstep-centrino$EXT ;;

	### Mobile Intel(R) Celeron(R) CPU 2.00GHz
	# According to: http://www.bay-wolf.com/speedstep.htm#10
	#
	# Q: Does the mobile Intel Celeron processor have the Intel SpeedStep
	# technology feature?
	#
	# A: No, mobile Intel Celeron processors do not have the Intel
	# SpeedStep technology feature. Intel SpeedStep technology is a
	# feature found on all mobile Intel Pentium 4 processors - M and
	# Pentium III processors - M at 600 MHz and above.
	Mobile\ Intel\(R\)\ Celeron\(R\)\ CPU*)
	    MODULE=none ;;

        Intel\(R\)\ Pentium\(R\)\ 4\ CPU*)
            if [ $LAPTOP -eq 1 ]; then
                MODULE=p4-clockmod$EXT
            fi
            ;;

        Intel\(R\)\ Pentium\(R\)\ 4\ Mobile\ Family\ CPU*)
                MODULE=$PIII_MODULE ;;
        esac;;
    AuthenticAMD*)
    case "$MODEL_NAME" in
	### AMD K7 diveratives
	[mM]obile\ AMD\ Athlon\(tm\)\ XP*)
	    MODULE=powernow-k7$EXT ;;
	[mM]obile\ AMD\ Duron\(tm\)*)
	    MODULE=powernow-k7$EXT ;;

	### AMD K8 diveratives
	AMD\ Athlon\ 64\ Processor*)
	    # -iDunno @ #debian-uk
	    MODULE=powernow-k8$EXT ;;
	AMD\ Athlon\(tm\)\ 64\ Processor*)
	    MODULE=powernow-k8$EXT ;;
        [Mm]obile\ AMD\ Sempron\(tm\)\ Processor*)
	    MODULE=powernow-k8$EXT ;;
        [Mm]obile\ AMD\ Sempron\ Processor*)
	    MODULE=powernow-k8$EXT ;;
        [Mm]obile\ AMD\ Athlon\(tm\)\ 64\ Processor*)
	    MODULE=powernow-k8$EXT ;;
        [Mm]obile\ AMD\ Athlon\ 64\ Processor*)
	    MODULE=powernow-k8$EXT ;;
    esac;;
esac

# I think PowerPC's use  /proc/cpuinfo 'cpu:' and just have a number
# NEED TO CHECK
case $CPU_NAME in
    FIXME)
	MODULE=FIXME ;;
esac

if [ "$MODULE" = "none" ] ; then
    echo This processor \"$MODEL_NAME\" is known _not_ to support power-saving. >&2
elif [ "$MODULE" = "p4-clockmod$EXT" ]; then
    echo P4 Clockmod is intentionally disabled >&2
    exit 1
elif [ "$MODULE" = "FIXME" ]; then
    echo No known cpufreq module for \"$MODEL_NAME\". Please add/report. >&2
    exit 1
elif [ -n "$MODULE" ] ; then
    echo $MODULE $OPTIONS
    exit 0
else
    echo No known cpufreq module for \"$MODEL_NAME\". Please add/report. >&2
    exit 1
fi

exit
